
# This file was generated by 'versioneer.py' (0.8+) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.7.4'
version_full = 'bc18a5b07b434e0fb07ae3c959b04f9a47f754e0'
def get_versions(default={}, verbose=False):
    return {'version': version_version, 'full': version_full}

